import 'zx/globals'
import { spinner } from 'zx/experimental'
import { g1, g2, g3, g4, g5, g6, g7, g8, txt } from './palette.mjs'
import { prompt, selectApp, yesNoPrompt } from './prompt.mjs'
import { opSignin } from './op-signin.mjs'
import page from './page.mjs'
import { debug } from './debug.mjs'
import { listApps } from './apps.mjs'

export default async function env() {
  page()
  cd(process.env.DISPENSE_MONOREPO_PATH)
  await spinner(g4('writing...'), () =>
    Promise.all([
      $`ENV=local pnpm -r dev:setup`,
      $`ENV=dev pnpm -r dev:setup`,
      /* Only api has a test env */
      $`ENV=test pnpm --filter=api dev:setup`,
      /* Only write production keys on demand to avoid overexposing sensitive keys */
      ...[
        process.env.ENV === 'staging' || process.env.ENV === 'prod'
          ? $`ENV=${process.env.ENV} pnpm -r dev:setup`
          : Promise.resolve(),
      ],
    ])
  )
  console.log(chalk.green('✔ ') + txt('env variables written to .env.*'))
}
/**
 * Should this command be called sync instead?
 *
 * The cli should not bake in any known information about individual
 * packages or apps in the workspace! Changes to an app should
 * never break monorepo tooling.
 *
 * This script _should_ either, detect the callsite and determine which
 * apps env to push or allow the user to select the package to run against.
 * Recursively pushing env variables to all packages is too destructive (opinion).
 * We should make users push env variables one app at a time.
 *
 * ❌ await $`pnpm -r exec `;
 *
 * ✅ dispense env push --web-app
 *
 * The problem then exists that the cli needs to know deployment details of
 * each app. Maybe we can provide service specific (vercel, render, etc.) tooling
 * for apps to consume in package.json scripts? Maybe we detect the deployment type
 * of an app by looking for its respective configuration files?
 * — Solution undecided
 * */
export async function envPush() {
  await opSignin()
  const app = await selectedApp()
  console.log(app)
  cd(`${process.env.DISPENSE_MONOREPO_PATH}/apps/${app}`)
  if (
    argv.env === 'prod' &&
    !(await yesNoPrompt({ question: 'warning: destructive — push variables to prod?' }))
  ) {
    txt('better safe than sorry. exiting')
    return
  }

  const environments = argv.env
    ? [argv.env]
    : [
        'dev',
        'staging',
        /**
         *  Do not include prod in default environment list!
         *  Production needs explicit selection and an additional confirmation step.
         * */
      ]
  const opVariables = await getOpVariables(environments)
  const target = await deployTarget()
  $.verbose = argv.debug
  if (target === 'unknown') {
    console.error('deploy target not found. exiting')
    process.exit(1)
  } else if (target === 'vercel') {
    const vercelEnvironments = {
      dev: 'preview',
      staging: 'preview staging',
      prod: 'production',
    }
    if (argv.debug) console.log(chalk.cyan(JSON.stringify(opVariables, null, 2)))
    if (opVariables.length > 60) {
      console.warn(
        'Vercel has a rate limit that prevents us from pushing too many keys at once. Try pushing one environment at a time with --env'
      )
      process.exit(1)
    }
    /**
     *  Vercel has no method for changing an existing variables value.
     *  You must first remove a key, then add a key. A bit scary if the add fails.
     * */
    page()
    const rmResults = await Promise.all(
      opVariables.map(async (v) => {
        const [env, branch = ''] = vercelEnvironments[
          v.environment
        ] /* Needs to be split; zx special string substitution wraps variables ($"var") */
          .split(' ')
        console.log(chalk.red('— ') + txt(v.key) + g1(' → ') + g3(env + ' ' + branch))
        return $`yes | vercel env rm ${v.key} ${env} ${branch}`.catch(() => undefined)
      })
    )
    debug(JSON.stringify(rmResults, null, 2))
    page()
    const addResults = await Promise.all(
      opVariables.map(async (v) => {
        const [env, branch = ''] = vercelEnvironments[
          v.environment
        ] /* Needs to be split; zx special string substitution wraps variables ($"var") */
          .split(' ')
        console.log(chalk.green('+ ') + txt(v.key) + g1(' → ') + g3(env + ' ' + branch))
        return $`echo ${v.value} | vercel env add ${v.key} ${env} ${branch}`.catch(() => undefined)
      })
    )
      .then(() => {
        console.log(
          chalk.green('✔ ') +
            txt(`env variables pushed to vercel [`) +
            `https://vercel.com/dispense/${app}/settings/environment-variables` +
            txt(']')
        )
      })
      .catch(() => {
        console.error('something went wrong. run again with flag --debug to investigate')
      })
    debug(JSON.stringify(addResults, null, 2))
  } else if (target === 'render') {
    txt("applications deploying to render aren't supported yet")
  }
}
/**
 * Returns name of select (by `--filter` or prompt) application
 * @returns {string}
 */
async function selectedApp() {
  return (await listApps()).includes(argv.filter) ? argv.filter : await selectApp()
}
/**
 *
 */
/**
 * Returns deploy target of selected (by `--filter` or prompt) application
 * @typedef {'vercel' | 'render' | 'unknown'} DeployTarget
 * @returns {Promise<DeployTarget>}
 */
async function deployTarget() {
  const isVercel = await $`ls vercel.json`
  /**
   * Renderfile doesn't exist, it's a placeholder until we figure out
   * a better way to detect that an app is deployed with Render
   * For now, `isRender` always equals `false`
   */
  const isRender = await $`ls Renderfile`
  return isVercel ? 'vercel' : isRender ? 'render' : 'unknown'
}
/**
 * An environment variable and its metadatas
 * @typedef {Object} Variable
 * @property {string} environment - the name of the environment
 * @property {string} key - the name of the variable
 * @property {value} value - the value of the variable
 */
/**
 * Returns an array of every 1password env variable in a vault.
 * @param {Array<string>} environments - a list of environments
 * @returns {Array<Variable>}
 */
async function getOpVariables(environments) {
  return Promise.all(
    environments.map((environment) =>
      $`ENV=${environment} op inject -i op.env`.then((proc) =>
        proc.stdout
          .split('\n')
          .map((entry) => {
            const [key, value] = entry.split(' = ')
            return key?.length && value?.length ? { environment, key, value } : null
          })
          .filter(Boolean)
      )
    )
  ).then((matrix) => matrix.flat())
}
