import 'zx/globals'
import { spinner } from 'zx/experimental'
import { g1, g2, g3, g4, g8, txt } from './palette.mjs'

import { prompt, yesNoPrompt } from './prompt.mjs'
import { opSignin } from './op-signin.mjs'
import page from './page.mjs'
import { debug } from './debug.mjs'
import writeEnv from './env.mjs'

const softwares = [
  {
    name: 'homebrew',
    find: () => $`brew --version`,
    install: () =>
      $`/bin/bash -c "$(curl -fsSL https://raw.githubusercontent.com/Homebrew/install/HEAD/install.sh)"`,
  },
  {
    name: 'github cli',
    find: () => $`gh --version`,
    install: () => $`brew install gh`,
  },
  {
    name: '1password cli',
    find: () => $`op --version`,
    install: () => $`brew install --cask 1password/tap/1password-cli`,
  },
  {
    name: '1password for mac',
    find: () => $`ls /Applications | grep 1Password`,
    install: () => $`brew install --cask 1password`,
    onInstallComplete: () => {
      page()
      return prompt({
        question:
          'turn on biometric unlock, then press enter ' +
          '[https://developer.1password.com/docs/cli/get-started/#turn-on-biometric-unlock]',
      })
    },
  },
  {
    name: 'vercel cli',
    find: () => $`vercel --version`,
    install: () => $`pnpm i -g vercel`,
  },
]

const writeMonorepoPath = async (path) => {
  await $`echo export DISPENSE_MONOREPO_PATH=${'"' + path + '"'} >> ~/.zshrc`

  await $`echo alias dmr=$DISPENSE_MONOREPO_PATH >> ~/.zshrc`

  console.log(txt('path saved to ') + g3('~/.zshrc ') + txt('aliased as ') + g3('dmr'))
}

export default async function setup() {
  page()

  const missingInstallations = await Promise.all(
    softwares.map((s) =>
      s
        .find()
        .then(() => ({ ...s, installed: true }))
        .catch(() => ({ ...s, installed: false }))
    )
  ).then((i) => i.filter((i) => !i.installed))

  if (missingInstallations.length) {
    console.log(chalk.yellow`⚠ ` + txt("you're missing some software\n"))
    console.log(
      missingInstallations
        .map((i) => (i.installed ? chalk.green('✔ ') : chalk.red('⨉ ') + g2(i.name)))
        .join('\n') + '\n'
    )
  }

  for (const i of missingInstallations) {
    if (!i.installed && (await yesNoPrompt({ question: `install ${i.name}?` }))) {
      page()

      $.verbose = true

      if (!debug(`debug enabled: ${i.name} install skipped`)) await i.install()

      $.verbose = false

      await i.onInstallComplete?.()
    }
  }

  const monorepo = await (process.env.DISPENSE_MONOREPO_PATH
    ? { status: 'configured', path: process.env.DISPENSE_MONOREPO_PATH }
    : spinner(txt('looking for the dispense monorepo, this may take some time...'), () =>
        $`find ~ -regex ".*/dispense/.git" -type d -print -quit -depth 2`
          .then((res) => ({
            status: 'found',
            path: res.stdout.split('/.git')[0],
          }))
          .catch(() => ({ status: 'not-found', path: '~/dispense' }))
      ))

  if (monorepo.status === 'found') {
    page()

    console.log(
      chalk.green('✔ ') + txt('dispense monorepo found at ') + g4(monorepo.path) + txt('\nnice.')
    )

    await writeMonorepoPath(monorepo.path)

    await $`sleep 1.5`
  } else if (monorepo.status === 'not-found') {
    page()

    console.log(txt("dispense monorepo not found, let's go get it"))

    await $`sleep 1.5`

    if (!(await $`gh auth status`.catch(() => false))) {
      await $`gh auth login`
    }

    await within(async () => {
      await $`cd ~`
      await spinner(g3('omg im cloning...'), () => $`gh repo clone dispenseapp/dispense`)
      await writeMonorepoPath(monorepo.path)
    })

    page()

    console.log(chalk.green('✔ ') + txt('monorepo cloned at ') + g3(monorepo.path))

    await $`sleep 1.5`
  }

  await opSignin()

  if (!(await $`op account list | grep dispenseinc.1password.com`.catch(() => false))) {
    page()
    if (await yesNoPrompt({ question: 'add dispense 1password account?' })) await $`op account add`
  }

  page()
  await writeEnv()

  if (monorepo.status === 'not-found' || monorepo.status === 'found') {
    console.log(
      txt("this is your first time cloning the monorepo. does that mean you're a new employee?\n") +
        g1("i can't wait to see what you create !\n") +
        txt('before you take off, pls ') +
        g4('source ~/.zshrc\n') +
        g8('now get out there and sell some weed~')
    )
  }
}
